#! /usr/bin/env python
#
# Look up words in phoneme dictionaries, like the "British English Example
# Pronounciations" file.
#
# Copyright (C) 2008 Siegfried-Angel Gevatter Pujals <siggi.gevatter@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import cPickle as pickle

cache_file = os.path.expanduser('~/.cache/phonemes-beep/dictionary')

def load(file):
	print 'Loading file into permanent data cache...'
	dictionary = {}
	for line in open(file).readlines():
		parts = [part.lower() for part in \
			line.replace('\t', ' ').split(' ') if part]
		if len(parts) < 2:
			continue
		elif len(parts) > 2:
			parts[1] = ' '.join(parts[1:])
		dictionary[parts[0]] =  parts[1].strip()
	if not os.path.isdir(os.path.dirname(cache_file)):
		try:
			os.mkdir(os.path.dirname(cache_file))
		except IOError:
			print 'Please create directory "%s" and run this again.' % \
				os.path.dirname(cache_file)
			sys.exit(1)
	pickle.dump(dictionary, open(cache_file, 'w'), -1)
	print 'Done.'

def lookup(dictionary, word):
	if word.lower() in dictionary:
		print dictionary[word.lower()]
	else:
		print 'Could not find "%s" in the data cache.' % word

def interactive(dictionary):
	print 'Interactive mode. Press Ctrl+C or introduce a blank line to exit.'
	print 'You make consider running this script with "rlwrap".\n'
	while 1:
		try:
			word = raw_input('Word: ')
		except EOFError:
			word = ''
		if not word:
			print
			sys.exit(0)
		else:
			lookup(dictionary, word)

def main():
	if '-h' in sys.argv or '--help' in sys.argv:
		print 'Usage: %s [<word>|--interactive]' % sys.argv[0]
		sys.exit(0)
	if not os.path.isfile(cache_file):
		print 'Please indicate the path to a plain-text phoneme dictionary.'
		print 'If you don\'t already have one, you can download "British' + \
			' English Example Pronounciations" from ftp://' + \
			'svr-ftp.eng.cam.ac.uk/pub/comp.speech/dictionaries/beep.tar.gz'
		file = ''
		while not os.path.isfile(file):
			file = raw_input('Filename: ')
		load(file)
	dictionary = pickle.load(open(cache_file))
	if '-i' in sys.argv or '--interactive' in sys.argv:
		interactive(dictionary)
	elif len(sys.argv) != 2:
		print 'Incorrect number of arguments.'
		sys.exit(1)
	else:
		lookup(dictionary, sys.argv[1])

if __name__ == '__main__':
	try:
		main()
	except KeyboardInterrupt:
		sys.exit(1)
